from data import connect
import pandas as pd
import numpy as np
from sklearn.decomposition import PCA, TruncatedSVD
from sklearn.cluster import MiniBatchKMeans
import seaborn as sns
import matplotlib.pyplot as plt



def to_matrix():
    """returns an adjacency matrix of publishers to publisher link frequency"""

    DB = connect()

    bias_map = pd.DataFrame([
        {'label' :'left', 'value' : 0},
        {'label' :'left-center', 'value' : 1},
        {'label' :'center', 'value' : 2},
        {'label' :'right-center', 'value' : 3},
        {'label' :'right', 'value' : 4},
        {'label' :'allsides', 'value' : -1},
    ])
    bias = DB.sql("""
        SELECT
            b.id
            ,b.label
            ,m.value
        FROM publisher_bias b
        JOIN bias_map m
        ON b.label = m.label
        WHERE value != -1
    """).df()

    pub = DB.sql("""
            select 
                p.id
                ,p.name
                ,p.url
                ,b.label
                ,b.value
            from publishers p
            left join bias b
            on b.id = p.id
    """).df()

    edges = DB.sql("""
        WITH total as (
            SELECT
                s.publisher_id as id
                ,COUNT(1) as stories
            FROM stories s
            GROUP BY 
                s.publisher_id
        ), p as (
            SELECT
                p.id
                ,stories
            FROM publishers p
            LEFT JOIN total t
            ON t.id = p.id
            WHERE t.stories >= 20
        ), cte as (
            SELECT 
                r.publisher_id as child_id
                ,s.publisher_id as parent_id
                ,count(1) as links
            FROM related_stories r
            JOIN stories s
            ON s.id = r.parent_id
            group by 
                s.publisher_id
                ,r.publisher_id
        )
        SELECT
            p.id as parent_id
            ,cte.child_id
            ,links
        FROM p
        left JOIN cte
        ON p.id = cte.parent_id
    """).df()

    adj = edges.pivot(index='parent_id', columns='child_id', values='links').fillna(0)

    
    out = pd.DataFrame(adj.index.values, columns=['id'])
    out = pd.merge(out, pub, how='left', on='id')

    pca = PCA(n_components=4)
    pca_out = pca.fit_transform(adj)

    svd = TruncatedSVD(n_components=2, n_iter=7, random_state=42)
    svd_out = svd.fit_transform(adj)

    x = svd_out[:, 0]
    y = svd_out[:, 1]

    x = pca_out[:, 0]
    y = pca_out[:, 1]
    sns.scatterplot(x=x, y=y)
    plt.show()

    kmeans = MiniBatchKMeans(n_clusters=5, random_state=0, batch_size=6, n_init="auto")
    pred = kmeans.fit_predict(pca_out)

    sns.scatterplot(x=x, y=y, hue=pred)
    plt.show()

    sns.scatterplot(x=pub['first'], y=pub['second'], hue=pub['bias'])
    plt.show()
